from json.decoder import JSONDecodeError
from time import sleep
from typing import Iterable, Optional, Union
from urllib.parse import urljoin
from warnings import warn

import requests

from .exceptions import (
    DarkSearchJSONDecodeException,
    DarkSearchPageNotFound,
    DarkSearchQuotaExceed,
    DarkSearchRequestException,
    DarkSearchServerError,
)
from .query import Query

BASE_URL = "https://darksearch.io"


def lookahead(iterable: Iterable):
    """Pass through all values from the given iterable, augmented by the
    information if there are more values to come after the current one
    (True), or if it is the last value (False).
    """
    it = iter(iterable)
    last = next(it)
    for val in it:
        yield last, True
        last = val
    yield last, False


class Client:
    """Client Class for DarkSearch API."""

    def __init__(
        self,
        base_url: str = BASE_URL,
        timeout: int = 30,
        headers: Optional[dict] = None,
        proxies: Optional[dict] = None,
    ):
        """Initialize this client with the given parameters.

        :param base_url: Base URL for the API
        :param timeout: Timeout for the requests
        :param headers: Headers for the request
        :param proxies: Proxies configuration
        :type base_url: str
        :type timeout: int
        :type headers: dict
        :type proxies: dict
        """
        self.base_url = base_url
        self.timeout = timeout
        self.headers = headers
        self.proxies = proxies

    def api_request(self, path: str, params: Optional[dict] = None, json: bool = True):
        """Perform an API request to DarkSearch.

        :param path: Path to be requested
        :param base_url: Base URL to be combined with the path
        :param params: Base URL to be combined with the path
        :type path: str
        :type base_url: str
        :type params: dict
        :return: JSON Response
        :rtype: dict
        """
        try:
            response = requests.get(
                urljoin(self.base_url, path),
                timeout=self.timeout,
                params=params,
                headers=self.headers,
                proxies=self.proxies,
            )

            if response.status_code == 404:
                raise DarkSearchPageNotFound
            if response.status_code == 429:
                raise DarkSearchQuotaExceed
            if response.status_code >= 500:
                raise DarkSearchServerError

            if json:
                return response.json()

            return response.content

        except requests.exceptions.RequestException as error:
            raise DarkSearchRequestException from error

        except JSONDecodeError:
            raise DarkSearchJSONDecodeException

    def api_search(self, query: str, page: int):
        """Perform low level search with the API.

        :param query: Query to be searched
        :param page: Page number of the search
        :type query: str
        :type page: int
        :return: search results
        :rtype: dict
        """

        if page < 1:
            page = 1
        return self.api_request("/api/search", params={"query": query, "page": page})

    def search(
        self,
        query: Union[str, Query],
        page: Optional[int] = None,
        pages: Optional[int] = None,
        wait: Optional[int] = None,
    ):
        """Perform search with the API.

        :param query: Query to be searched
        :param page: Page number of the search
        :param pages: Pages to be returned
        :param wait: Time to wait between requests
        :type query: str | Query
        :type page: int
        :type pages: int
        :type wait: int
        :return: search results
        :rtype: list or dict
        """

        # Converts Query object to string
        if isinstance(query, Query):
            query = str(query)

        # Multiple pages
        if pages and not page:
            results = list()
            try:
                for page_i, has_more in lookahead(range(pages)):
                    response = self.api_search(query, page_i + 1)
                    results.append(response)
                    if response.get("last_page") <= page_i + 1:
                        break
                    if wait and has_more:
                        sleep(wait)
            except DarkSearchQuotaExceed:
                warn(
                    "Search Quota Exceeded, please keep searches to less than "
                    "30 per minute"
                )
            return results
        if not page:
            page = 1

        # Single page
        return self.api_search(query, page)

    def crawling_status(self):
        """Return the number of indexed pages in DarkSearch.

        :return: crawling_status
        :rtype: int
        """
        return int(self.api_request("/api/crawling_status", json=False))



from typing import List


class Query:
    def __init__(self, query: str, exact: bool = False):
        self._query_list: List[str] = []
        self.add_query(query, exact=exact)

    def __str__(self):
        return " ".join(self._query_list)

    def add_query(self, string: str, exact: bool = False):
        if exact:
            string = '"' + string + '"'
        self._query_list.append(string)
        return self

    def and_(self, query: str, exact: bool = False):
        self.add_query("AND")
        return self.add_query(query.strip(), exact=exact)

    def or_(self, query: str, exact: bool = False):
        self.add_query("OR")
        return self.add_query(query.strip(), exact=exact)

    def not_(self, query: str, exact: bool = False):
        self.add_query("NOT")
        return self.add_query(query.strip(), exact=exact)



from argparse import ArgumentParser
from pprint import pprint
from typing import List

from rich import print
from rich.markdown import Markdown
from rich.panel import Panel
from rich.text import Text
from rich.tree import Tree

from .api import Client


def print_response(responses: List[dict]):
    """Print the response from the API.

    :param response: the API response
    :type response: dict
    """
    for response in responses:
        page_tree = Tree("Page {}\n".format(response.get("current_page")))
        page_tree.add("From: {}".format(response.get("from")))
        page_tree.add("To: {}".format(response.get("to")))
        page_tree.add("Per Page: {}".format(response.get("per_page")))
        page_tree.add("Last Page: {}\n".format(response.get("last_page")))
        results_tree = page_tree.add("Results")
        for result in response.get("data", []):
            result_branch = results_tree.add(Text(result.get("title"), "purple"))
            description_branch = result_branch.add("📜 Description")
            description_branch.add(Panel(Markdown(result.get("description"))))
            link_branch = result_branch.add("🔗 Link")
            link_branch.add(result.get("link"))

    print(results_tree)


def get_parser():
    """Returns ArgumentParser"""
    parser = ArgumentParser(description="DarkSearch API Client")
    parser.add_argument("query", help="search query")
    parser.add_argument("-p", "--page", type=int, help="page number")
    parser.add_argument("-n", "--pages", type=int, help="number of pages")
    parser.add_argument("-w", "--wait", type=int, help="wait between requests")
    parser.add_argument("-j", "--json", action="store_true", help="prints as json")
    parser.add_argument(
        "-v", "--verbose", action="store_true", help="prints verbose json"
    )
    return parser


def main():
    """Execute this command line."""
    parser = get_parser()
    args = parser.parse_args()

    if not args.query:
        parser.print_help()
        exit(1)

    client = Client(timeout=10)

    response = client.search(
        args.query, page=args.page, pages=args.pages, wait=args.wait
    )

    if args.json:
        print_func = print
        if not args.verbose:
            print_func = pprint
        print_func(response)
    else:
        if not isinstance(response, list):
            response = [response]
        print_response(response)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['darksearch']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.26.0,<3.0.0',
 'rich>=10.15.1,<11.0.0',
 'types-requests>=2.26.1,<3.0.0']

entry_points = \
{'console_scripts': ['darksearch = darksearch.cli:main']}

setup_kwargs = {
    'name': 'darksearch',
    'version': '2.1.3',
    'description': 'Python API wrapper for DarkSearch (darksearch.io).',
    'long_description': '# DarkSearch\n\n[![PyPI](https://img.shields.io/pypi/v/darksearch?color=orange&logo=pypi&logoColor=orange)](https://pypi.org/project/darksearch/)\n[![Python Version](https://img.shields.io/badge/python-3.6%2B-blue?logo=python)](https://www.python.org/downloads/)\n[![GitHub Workflow Status](https://img.shields.io/github/workflow/status/thehappydinoa/DarkSearch/test?label=tests)](https://github.com/thehappydinoa/DarkSearch/actions)\n[![Read the Docs](https://img.shields.io/readthedocs/darksearch/latest)](https://censys-python.readthedocs.io/en/stable/?badge=stable)\n[![License](https://img.shields.io/github/license/thehappydinoa/DarkSearch)](LICENSE)\n\nPython API wrapper for DarkSearch ([darksearch.io](https://darksearch.io/)). Python 3.6+ is currently supported.\n\n## Install\n\n```bash\npip install darksearch\n```\n\n## Resources\n\n- [Documentation](https://darksearch.readthedocs.io/)\n\n## Contributing\n\nAll contributions (no matter how small) are always welcome.\n\n## Development\n\n```bash\ngit clone git@github.com:thehappydinoa/DarkSearch.git\npoetry install\n```\n\n## Testing\n\n```bash\npytest\n```\n\n## Publishing\n\n```bash\npoetry build\npoetry publish\n```\n\n## License\n\nThis software is licensed under [MIT](LICENSE)\n',
    'author': 'Aidan Holland',
    'author_email': 'thehappydinoa@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/thehappydinoa/DarkSearch',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.2,<4',
}


setup(**setup_kwargs)
